#include "stdafx.h"
#include "clf_camera.h"
#include "clf_light.h"
#include "clf_shader.h"
#include "clf_skybox.h"
#include "clf_effects.h"
#include "clf_world.h"
#include "clf_mesh.h"
#include "clf_font.h"
#include "clf_util.h"

#include "clf_application.h"
#include "clf_menu.h"
#include "clf_input.h"

class wait_handler : public clf_dispatch {
public:
	cCamera m_cam;	// menu camera
	clf_stage m_stage;
	clf_light m_light;	// menu light

	wait_handler() : clf_dispatch(){
	}
	virtual void window_resize (int width, int height){			// reshape the window when it's moved or resized
		m_stage.resize(0,0,width, height);
	}
	virtual bool graphics_init(){
		get_font_manager().create_font_3d("Bookman3D", "Bookman Old Style", 20, .2f);

		// set up menu light and camera
		m_cam.setup(clf_vector3D(0,0,-7),       // position viewer
			clf_vector3D(0,0,0),        // setup the point of interest (where we are looking at)
			clf_vector3D(0,1,0));       // which way is up

		// Light model parameters:
		// -------------------------------------------
		// create and position a light source
		m_light.set_ambient(0.25f, 0.25f, 0.25f, 1.f);
		m_light.set_diffuse(0.7f, 0.7f, 0.7f, 1.f);
		m_light.set_specular(1.f, 1.f, 1.f, 1.f);
		m_light.set_position(0.f, 0.f, -100.f, 0.f);
		return true;
	}
	virtual void graphics_terminate(){
	}
	virtual void frame_move(clFloat fElapsed){
	}
	virtual void frame_render(){
		// clear the screen
		// drawing the skybox means we do not have to waste time clearing the screen
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	// clear screen and stencil buffer

		glMatrixMode (GL_PROJECTION);									// select the projection matrix
		glLoadIdentity ();														// reset the projection matrix

		// viewport and perspective
		m_stage.prepare();

		glMatrixMode (GL_MODELVIEW);								// select the modelview matrix
		glLoadIdentity ();													// reset

		glPushMatrix();

		// position the viewer
		m_cam.apply();

		glPushAttrib(GL_ALL_ATTRIB_BITS);

		glEnable(GL_NORMALIZE);
		// position the light
		m_light.enable();

		// reapply the light
		m_light.apply();

		clf_font* pfont = get_font_manager().get_font("Bookman3D");
		pfont->set_color_RGB(0.f,0.f,.95f);
		pfont->print_3d(ALIGN_CENTER, "initializing...");

		glPopMatrix();
		glPopAttrib();
	}
};

class theGame : public clf_application
{
	clf_dispatch* current_dispatch;
	wait_handler* m_wh;
public:
	theGame() {
		m_wh = new wait_handler();
		current_dispatch = m_wh;
		win.init.title			= "clf_samples";				// Window Title
		win.init.width			= 1680;									// Full Screen Width
		win.init.height			= 1050;									// Full Screen Height
		win.init.bitsPerPixel	= 32;									// Bits Per Pixel
		win.init.w_width    = 840;									// Window Width
		win.init.w_height   = 525;									// Window Height
		win.init.isFullScreen	= FALSE;							// Fullscreen? (Set To TRUE)
		m_nFPS = 0;
		m_nFrames = 0;											// FPS and FPS Counter
		g_pause=true;
		g_startup_timer = 2.f;
	}
	virtual ~theGame(){
	}

	int m_nFPS;
	int m_nFrames;											// FPS and FPS Counter
	bool g_pause;
	clFloat g_startup_timer;

	clf_menu  m_menu;

	virtual void window_resize (int width, int height);			// reshape the window when it's moved or resized
	virtual bool graphics_init();
	virtual void graphics_terminate();
	virtual void frame_move(clFloat fElapsed);
	virtual void frame_render();

	void menu_cback(int opt);
};

theGame g_game;

void theGame::menu_cback(int opt)
{
	// print("selected option:%d", opt);
	if (opt == 1)
	{
		g_startup_timer = 2.f;
		current_dispatch = m_wh;
	}
	else if (opt == 2)
	{
		terminate_application ();
	}
}

void theGame::window_resize (int width, int height)			// reshape the window when it's moved or resized
{
	m_wh->window_resize(width, height);
	m_menu.window_resize(width, height);
}

bool theGame::graphics_init()
{
	m_wh->graphics_init();

	glEnable (GL_DEPTH_TEST);        // enable depth test
	glEnable(GL_COLOR_MATERIAL);     // use color for drawing the object

	get_font_manager().create_font_2d("Bookman2D", "Bookman Old Style", 16);
	get_font_manager().create_font_3d("Bookman20", "Bookman Old Style", 20, 0.2f);

	static bool menu_set = false;
	if (!menu_set) // set only once
	{
		static clf_functor_inst<theGame> callback_fkt(this, &theGame::menu_cback);
		m_menu.add_option("Repeat", 1);
		m_menu.add_option("Exit", 2);
		m_menu.set_pos(0,0,0);
		m_menu.set_menu_callBack(&callback_fkt);
		menu_set = true;
	}

	m_menu.graphics_init();
	

	return true;
}

void theGame::graphics_terminate()
{
	m_wh->graphics_terminate();
	m_menu.graphics_terminate();
	get_font_manager().clear();
}

void theGame::frame_move(clFloat fElapsed)
{
	clf_application::frame_move(fElapsed);

	if (isKeyPressed(DIK_P))
	{
		g_pause = !g_pause;
		g_startup_timer = -1.f;
	}

	if (g_startup_timer > 0.f)
	{
		g_startup_timer -= fElapsed;
		if (g_startup_timer <= 0.f)
		{
			g_pause = false;
			current_dispatch = &m_menu;
		}
	}

	if (!g_pause)
	{
		current_dispatch->frame_move(fElapsed);
	}

	// frame counting mechanism
	// the higher the frame rate the faster the system
	// if frame rate drops below 30 we are in deep trouble
	// we should either optimize the program or buy a new computer
	static clFloat tot = 0;
	tot += fElapsed;
	m_nFrames++;												// Increment Our frame Counter
	if (tot >= 1.f)
	{
		tot = 0;
		m_nFPS = m_nFrames;
		m_nFrames = 0;

		char txt[200];
		sprintf(txt, "GusOnGames, fps:%d",m_nFPS);
		SetWindowText(win.hWnd, txt);
	}
}

void theGame::frame_render()
{
	current_dispatch->frame_render();
}
